@@ -9,7 +9,6 @@
 #include "MathUtils.h"
 #include "utils/StringUtils.h"
 
-
 namespace OCIO_NAMESPACE
 {
 // This method converts a float/double to a string adding a dot when
@@ -44,6 +43,8 @@ std::string getVecKeyword(GpuLanguage lang)
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         {
             kw << "vec" << N;
             break;
@@ -53,11 +54,18 @@ std::string getVecKeyword(GpuLanguage lang)
             kw << "half" << N;
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+
+        case GPU_LANGUAGE_MSL_2_0:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
             kw << "float" << N;
             break;
         }
+        case LANGUAGE_OSL_1:
+        {
+            kw << "vector" << N;
+            break;
+        }
 
         default:
         {
@@ -80,6 +88,8 @@ void getTexDecl(GpuLanguage lang,
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_CG:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         {
             textureDecl = "";
 
@@ -88,7 +98,7 @@ void getTexDecl(GpuLanguage lang,
             samplerDecl = kw.str();
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
             std::ostringstream t;
             t << "Texture" << N << "D " << textureName << ";";
@@ -99,6 +109,21 @@ void getTexDecl(GpuLanguage lang,
             samplerDecl = t.str();
             break;
         }
+        case LANGUAGE_OSL_1:
+        {
+            throw Exception("Unsupported by the Open Shading language (OSL) translation.");
+        }
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            std::ostringstream t;
+            t << "texture" << N << "d<float> " << textureName << ";";
+            textureDecl = t.str();
+
+            t.str("");
+            t << "sampler" << " " << samplerName << ";";
+            samplerDecl = t.str();
+            break;
+        }
 
         default:
         {
@@ -118,8 +143,21 @@ std::string getTexSample(GpuLanguage lang,
     switch (lang)
     {
         case GPU_LANGUAGE_GLSL_1_2:
+        {
+            kw << "texture" << N << "D(" << samplerName << ", " << coords << ")";
+            break;
+        }
         case GPU_LANGUAGE_GLSL_1_3:
         {
+            kw << "texture(" << samplerName << ", " << coords << ")";
+            break;
+        }
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        {
+            if (N == 1) {
+                throw Exception("1D textures are unsupported by OpenGL ES.");
+            }
+
             kw << "texture" << N << "D(" << samplerName << ", " << coords << ")";
             break;
         }
@@ -128,7 +166,7 @@ std::string getTexSample(GpuLanguage lang,
             kw << "tex" << N << "D(" << samplerName << ", " << coords << ")";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
             kw << textureName << ".Sample(" << samplerName << ", " << coords << ")";
             break;
@@ -138,7 +176,25 @@ std::string getTexSample(GpuLanguage lang,
             kw << "texture(" << samplerName << ", " << coords << ")";
             break;
         }
+        case GPU_LANGUAGE_GLSL_ES_3_0:
+        {
+            if (N == 1) {
+                throw Exception("1D textures are unsupported by OpenGL ES.");
+            }
 
+            kw << "texture(" << samplerName << ", " << coords << ")";
+            break;
+        }
+        case LANGUAGE_OSL_1:
+        {
+            throw Exception("Unsupported by the Open Shading language (OSL) translation.");
+        }
+
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            kw << textureName << ".sample(" << samplerName << ", " << coords << ")";
+            break;
+        }
         default:
         {
             throw Exception("Unknown GPU shader language.");
@@ -262,7 +318,7 @@ std::string GpuShaderText::string() const
 
 void GpuShaderText::flushLine()
 {
-    static const unsigned tabSize = 2;
+    static constexpr unsigned tabSize = 2;
 
     m_ossText << std::string(tabSize * m_indent, ' ')
               << m_ossLine.str()
@@ -272,33 +328,177 @@ void GpuShaderText::flushLine()
     m_ossLine.clear();
 }
 
+std::string GpuShaderText::constKeyword() const
+{
+    std::string str;
+
+    switch (m_lang)
+    {
+        case GPU_LANGUAGE_GLSL_1_2:
+        case GPU_LANGUAGE_GLSL_1_3:
+        case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            str += "const";
+            str += " ";
+            break;
+        }
+        case GPU_LANGUAGE_HLSL_SM_5_0:
+        {
+            str += "static const";
+            str += " ";
+            break;
+        }
+        case LANGUAGE_OSL_1:
+        case GPU_LANGUAGE_CG:
+            break;
+    }
+
+    return str;
+}
+
 std::string GpuShaderText::floatKeyword() const
 {
     return (m_lang == GPU_LANGUAGE_CG ? "half" : "float");
 }
 
+std::string GpuShaderText::floatKeywordConst() const
+{
+    std::string str;
+
+    str += constKeyword();
+    str += floatKeyword();
+
+    return str;
+}
+
+std::string GpuShaderText::floatDecl(const std::string & name) const
+{
+    if (name.empty())
+    {
+        throw Exception("GPU variable name is empty.");
+    }
+
+    return floatKeyword() + " " + name;
+}
+
 std::string GpuShaderText::intKeyword() const
 {
     return "int";
 }
 
-void GpuShaderText::declareVar(const std::string & name, float v)
+std::string GpuShaderText::intKeywordConst() const
+{
+    std::string str;
+
+    str += constKeyword();
+    str += intKeyword();
+
+    return str;
+}
+
+std::string GpuShaderText::intDecl(const std::string & name) const
 {
     if (name.empty())
     {
         throw Exception("GPU variable name is empty.");
     }
-    newLine() << floatKeyword()
-              << " " << name << " = " << getFloatString(v, m_lang) << ";";
+
+    return intKeyword() + " " + name;
+}
+
+std::string GpuShaderText::colorDecl(const std::string & name) const
+{
+    if (name.empty())
+    {
+        throw Exception("GPU variable name is empty.");
+    }
+
+    return (m_lang==LANGUAGE_OSL_1 ? "color" : float3Keyword()) + " " + name;
+}
+
+void GpuShaderText::declareVarConst(const std::string & name, float v)
+{
+    newLine() << constKeyword() << declareVarStr(name, v) << ";";
+}
+
+void GpuShaderText::declareVar(const std::string & name, float v)
+{
+    newLine() << declareVarStr(name, v) << ";";
+}
+
+// TODO: OSL: The method only solves the problem for constant float values. The code must also
+// support the in-place declarations (like res = t + vec3(...) for example).
+
+std::string GpuShaderText::declareVarStr(const std::string & name, float v)
+{
+    if (name.empty())
+    {
+        throw Exception("GPU variable name is empty.");
+    }
+
+    // Note that OSL does not support inf and -inf so the code below adjusts the value to float max.
+
+    if (std::isinf(v))
+    {
+        float newVal = v;
+
+        if (std::signbit(v))
+        {
+            newVal = -1 * std::numeric_limits<float>::max();
+        }
+        else
+        {
+            newVal = std::numeric_limits<float>::max();
+        }
+
+        std::ostringstream oss;
+        oss.precision(std::numeric_limits<float>::max_digits10);
+        oss << newVal;
+
+        return floatDecl(name) + " = " + oss.str();
+    }
+
+    return floatDecl(name) + " = " + getFloatString(v, m_lang);
+}
+
+std::string GpuShaderText::vectorCompareExpression(const std::string& lhs, const std::string& op, const std::string& rhs)
+{
+    std::string ret = lhs + " " + op + " " + rhs;
+    if(m_lang == GPU_LANGUAGE_MSL_2_0)
+    {
+        ret = "any( " + ret + " )";
+    }
+    return ret;
+}
+
+void GpuShaderText::declareVarConst(const std::string & name, bool v)
+{
+    newLine() << constKeyword() << declareVarStr(name, v) << ";";
 }
 
 void GpuShaderText::declareVar(const std::string & name, bool v)
+{
+    newLine() << declareVarStr(name, v) << ";";
+}
+
+std::string GpuShaderText::declareVarStr(const std::string & name, bool v)
 {
     if (name.empty())
     {
         throw Exception("GPU variable name is empty.");
     }
-    newLine() << "bool " << name << " = " << (v ? "true;" : "false;");
+
+    if (m_lang==LANGUAGE_OSL_1)
+    {
+        return intKeyword() + " " + name + " = " + (v ? "1" : "0");
+    }
+    else
+    {
+        return "bool " + name + " = " + (v ? "true" : "false");
+    }
 }
 
 void GpuShaderText::declareFloatArrayConst(const std::string & name, int size, const float * v)
@@ -319,8 +519,10 @@ void GpuShaderText::declareFloatArrayConst(const std::string & name, int size, c
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         {
-            nl << "const " << floatKeyword() << " " << name << "[" << size << "] = ";
+            nl << floatKeywordConst() << " " << name << "[" << size << "] = ";
             nl << floatKeyword() << "[" << size << "](";
             for (int i = 0; i < size; ++i)
             {
@@ -333,23 +535,12 @@ void GpuShaderText::declareFloatArrayConst(const std::string & name, int size, c
             nl << ");";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
-        {
-            nl << "const " << floatKeyword() << " " << name << "[" << size << "] = {";
-            for (int i = 0; i < size; ++i)
-            {
-                nl << getFloatString(v[i], m_lang);
-                if (i + 1 != size)
-                {
-                    nl << ", ";
-                }
-            }
-            nl << "};";
-            break;
-        }
+        case LANGUAGE_OSL_1:
         case GPU_LANGUAGE_CG:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
+        case GPU_LANGUAGE_MSL_2_0:
         {
-            nl << floatKeyword() << " " << name << "[" << size << "] = {";
+            nl << floatKeywordConst() << " " << name << "[" << size << "] = {";
             for (int i = 0; i < size; ++i)
             {
                 nl << getFloatString(v[i], m_lang);
@@ -382,8 +573,10 @@ void GpuShaderText::declareIntArrayConst(const std::string & name, int size, con
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         {
-            nl << "const " << intKeyword() << " " << name << "[" << size << "] = "
+            nl << intKeywordConst() << " " << name << "[" << size << "] = "
                << intKeyword() << "[" << size << "](";
             for (int i = 0; i < size; ++i)
             {
@@ -396,9 +589,10 @@ void GpuShaderText::declareIntArrayConst(const std::string & name, int size, con
             nl << ");";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
+        case GPU_LANGUAGE_MSL_2_0:
         {
-            nl << "const " << intKeyword() << " " << name << "[" << size << "] = {";
+            nl << intKeywordConst() << " " << name << "[" << size << "] = {";
             for (int i = 0; i < size; ++i)
             {
                 nl << v[i];
@@ -410,6 +604,7 @@ void GpuShaderText::declareIntArrayConst(const std::string & name, int size, con
             nl << "};";
             break;
         }
+        case LANGUAGE_OSL_1:
         case GPU_LANGUAGE_CG:
         {
             nl << intKeyword() << " " << name << "[" << size << "] = {";
@@ -432,6 +627,13 @@ std::string GpuShaderText::float2Keyword() const
     return getVecKeyword<2>(m_lang);
 }
 
+std::string GpuShaderText::float2Const(const std::string& x, const std::string& y) const
+{
+    std::ostringstream kw;
+    kw << float2Keyword() << "(" << x << ", " << y << ")";
+    return kw.str();
+}
+
 std::string GpuShaderText::float2Decl(const std::string & name) const
 {
     if (name.empty())
@@ -444,26 +646,26 @@ std::string GpuShaderText::float2Decl(const std::string & name) const
 
 std::string GpuShaderText::float3Keyword() const
 {
-    return getVecKeyword<3>(m_lang);
+    return m_lang == LANGUAGE_OSL_1 ? "vector" : getVecKeyword<3>(m_lang);
 }
 
 std::string GpuShaderText::float3Const(float x, float y, float z) const
 {
     return float3Const(getFloatString(x, m_lang),
-                      getFloatString(y, m_lang), 
-                      getFloatString(z, m_lang));
+                       getFloatString(y, m_lang), 
+                       getFloatString(z, m_lang));
 }
 
 std::string GpuShaderText::float3Const(double x, double y, double z) const
 {
     return float3Const(getFloatString(x, m_lang),
-                      getFloatString(y, m_lang), 
-                      getFloatString(z, m_lang));
+                       getFloatString(y, m_lang), 
+                       getFloatString(z, m_lang));
 }
 
 std::string GpuShaderText::float3Const(const std::string& x,
-                                      const std::string& y,
-                                      const std::string& z) const
+                                       const std::string& y,
+                                       const std::string& z) const
 {
     std::ostringstream kw;
     kw << float3Keyword() << "(" << x << ", " << y << ", " << z << ")";
@@ -495,12 +697,11 @@ std::string GpuShaderText::float3Decl(const std::string &  name) const
     return float3Keyword() + " " + name;
 }
 
-void GpuShaderText::declareFloat3(const std::string & name,
-                                 float x, float y, float z)
+void GpuShaderText::declareFloat3(const std::string & name, float x, float y, float z)
 {
     declareFloat3(name, getFloatString(x, m_lang), 
-                       getFloatString(y, m_lang), 
-                       getFloatString(z, m_lang));
+                        getFloatString(y, m_lang), 
+                        getFloatString(z, m_lang));
 }
 
 void GpuShaderText::declareFloat3(const std::string& name, const Float3 & vec3)
@@ -509,17 +710,17 @@ void GpuShaderText::declareFloat3(const std::string& name, const Float3 & vec3)
 }
 
 void GpuShaderText::declareFloat3(const std::string & name, 
-                                 double x, double y, double z)
+                                  double x, double y, double z)
 {
     declareFloat3(name, getFloatString(x, m_lang), 
-                       getFloatString(y, m_lang), 
-                       getFloatString(z, m_lang));
+                        getFloatString(y, m_lang), 
+                        getFloatString(z, m_lang));
 }
 
 void GpuShaderText::declareFloat3(const std::string & name, 
-                                 const std::string & x,
-                                 const std::string & y,
-                                 const std::string & z)
+                                  const std::string & x,
+                                  const std::string & y,
+                                  const std::string & z)
 {
     newLine() << float3Decl(name) << " = " << float3Const(x, y, z) << ";";
 }
@@ -532,23 +733,23 @@ std::string GpuShaderText::float4Keyword() const
 std::string GpuShaderText::float4Const(const float x, const float y, const float z, const float w) const
 {
     return float4Const(getFloatString(x, m_lang),
-                      getFloatString(y, m_lang), 
-                      getFloatString(z, m_lang),
-                      getFloatString(w, m_lang));
+                       getFloatString(y, m_lang), 
+                       getFloatString(z, m_lang),
+                       getFloatString(w, m_lang));
 }
 
 std::string GpuShaderText::float4Const(const double x, const double y, const double z, const double w) const
 {
     return float4Const(getFloatString(x, m_lang),
-                      getFloatString(y, m_lang), 
-                      getFloatString(z, m_lang),
-                      getFloatString(w, m_lang));
+                       getFloatString(y, m_lang), 
+                       getFloatString(z, m_lang),
+                       getFloatString(w, m_lang));
 }
 
-std::string GpuShaderText::float4Const(const std::string& x,
-                                      const std::string& y,
-                                      const std::string& z,
-                                      const std::string& w) const
+std::string GpuShaderText::float4Const(const std::string & x,
+                                       const std::string & y,
+                                       const std::string & z,
+                                       const std::string & w) const
 {
     std::ostringstream kw;
     kw << float4Keyword() << "("
@@ -687,27 +888,93 @@ std::string GpuShaderText::sampleTex3D(const std::string& textureName,
 
 void GpuShaderText::declareUniformFloat(const std::string & uniformName)
 {
-    newLine() << "uniform " << floatKeyword() << " " << uniformName << ";";
+    newLine() << (m_lang == GPU_LANGUAGE_MSL_2_0 ? "" : "uniform ") << floatKeyword() << " " << uniformName << ";";
 }
 
 void GpuShaderText::declareUniformBool(const std::string & uniformName)
 {
-    newLine() << "uniform bool " << uniformName << ";";
+    newLine() << (m_lang == GPU_LANGUAGE_MSL_2_0 ? "" : "uniform ") << "bool " << uniformName << ";";
 }
 
 void GpuShaderText::declareUniformFloat3(const std::string & uniformName)
 {
-    newLine() << "uniform " << float3Keyword() << " " << uniformName << ";";
+    newLine() << (m_lang == GPU_LANGUAGE_MSL_2_0 ? "" : "uniform ") << float3Keyword() << " " << uniformName << ";";
 }
 
 void GpuShaderText::declareUniformArrayFloat(const std::string & uniformName, unsigned int size)
 {
-    newLine() << "uniform " << floatKeyword() << " " << uniformName << "[" << size << "];";
+    newLine() << (m_lang == GPU_LANGUAGE_MSL_2_0 ? "" : "uniform ") << floatKeyword() << " " << uniformName << "[" << size << "];";
 }
 
 void GpuShaderText::declareUniformArrayInt(const std::string & uniformName, unsigned int size)
 {
-    newLine() << "uniform " << intKeyword() << " " << uniformName << "[" << size << "];";
+    newLine() << (m_lang == GPU_LANGUAGE_MSL_2_0 ? "" : "uniform ") << intKeyword() << " " << uniformName << "[" << size << "];";
+}
+
+// Keep the method private as only float & double types are expected
+template<typename T>
+std::string matrix3Mul(const T * m3x3, const std::string & vecName, GpuLanguage lang)
+{
+    if (vecName.empty())
+    {
+        throw Exception("GPU variable name is empty.");
+    }
+
+    std::ostringstream kw;
+    switch (lang)
+    {
+        case GPU_LANGUAGE_GLSL_1_2:
+        case GPU_LANGUAGE_GLSL_1_3:
+        case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
+        {
+            // OpenGL shader program requests a transposed matrix
+            kw << "mat3("
+                << getMatrixValues<T, 3>(m3x3, lang, true) << ") * " << vecName;
+            break;
+        }
+        case GPU_LANGUAGE_CG:
+        {
+            kw << "mul(half3x3("
+                << getMatrixValues<T, 3>(m3x3, lang, false) << "), " << vecName << ")";
+            break;
+        }
+        case GPU_LANGUAGE_HLSL_SM_5_0:
+        {
+            kw << "mul(" << vecName
+                << ", float3x3(" << getMatrixValues<T, 3>(m3x3, lang, true) << "))";
+            break;
+        }
+        case LANGUAGE_OSL_1:
+        {
+            kw << "matrix(" << getMatrixValues<T, 3>(m3x3, lang, false) << ") * " << vecName;
+            break;
+        }
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            kw << "float3x3(" << getMatrixValues<T, 3>(m3x3, lang, true) << ") * " << vecName;
+            break;
+        }
+
+        default:
+        {
+            throw Exception("Unknown GPU shader language.");
+        }
+    }
+    return kw.str();
+}
+
+std::string GpuShaderText::mat3fMul(const float * m3x3,
+                                    const std::string & vecName) const
+{
+    return matrix3Mul<float>(m3x3, vecName, m_lang);
+}
+
+std::string GpuShaderText::mat3fMul(const double * m3x3,
+                                    const std::string & vecName) const
+{
+    return matrix3Mul<double>(m3x3, vecName, m_lang);
 }
 
 // Keep the method private as only float & double types are expected
@@ -725,6 +992,8 @@ std::string matrix4Mul(const T * m4x4, const std::string & vecName, GpuLanguage
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         {
             // OpenGL shader program requests a transposed matrix
             kw << "mat4(" 
@@ -737,12 +1006,22 @@ std::string matrix4Mul(const T * m4x4, const std::string & vecName, GpuLanguage
                 << getMatrixValues<T, 4>(m4x4, lang, false) << "), " << vecName << ")";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
             kw << "mul(" << vecName 
                 << ", float4x4(" << getMatrixValues<T, 4>(m4x4, lang, true) << "))";
             break;
         }
+        case LANGUAGE_OSL_1:
+        {
+            kw << "matrix(" << getMatrixValues<T, 4>(m4x4, lang, false) << ") * " << vecName;
+            break;
+        }
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            kw << "float4x4(" << getMatrixValues<T, 4>(m4x4, lang, true) << ") * " << vecName;
+            break;
+        }
 
         default:
         {
@@ -771,15 +1050,19 @@ std::string GpuShaderText::lerp(const std::string & x,
     std::ostringstream kw;
     switch (m_lang)
     {
+        case LANGUAGE_OSL_1:
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
+        case GPU_LANGUAGE_MSL_2_0:
         {
             kw << "mix(" << x << ", " << y << ", " << a << ")";
             break;
         }
         case GPU_LANGUAGE_CG:
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
             kw << "lerp(" << x << ", " << y << ", " << a << ")";
             break;
@@ -802,15 +1085,21 @@ std::string GpuShaderText::float3GreaterThan(const std::string & a,
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         case GPU_LANGUAGE_CG:
         {
             kw << float3Keyword() << "(greaterThan( " << a << ", " << b << "))";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case LANGUAGE_OSL_1:
+        case GPU_LANGUAGE_MSL_2_0:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
-            kw << float3Keyword() << "((" << a << " > " << b << ") ? "
-                << float3Const(1.0f) << " : " << float3Const(0.0f) << ")";
+            kw << float3Keyword() << "(" 
+               << "(" << a << "[0] > " << b << "[0]) ? 1.0 : 0.0, "
+               << "(" << a << "[1] > " << b << "[1]) ? 1.0 : 0.0, "
+               << "(" << a << "[2] > " << b << "[2]) ? 1.0 : 0.0)";
             break;
         }
 
@@ -831,15 +1120,30 @@ std::string GpuShaderText::float4GreaterThan(const std::string & a,
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         case GPU_LANGUAGE_CG:
         {
             kw << float4Keyword() << "(greaterThan( " << a << ", " << b << "))";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_MSL_2_0:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
-            kw << float4Keyword() << "((" << a << " > " << b << ") ? "
-                << float4Const(1.0f) << " : " << float4Const(0.0f) << ")";
+            kw << float4Keyword() << "(" 
+               << "(" << a << "[0] > " << b << "[0]) ? 1.0 : 0.0, "
+               << "(" << a << "[1] > " << b << "[1]) ? 1.0 : 0.0, "
+               << "(" << a << "[2] > " << b << "[2]) ? 1.0 : 0.0, "
+               << "(" << a << "[3] > " << b << "[3]) ? 1.0 : 0.0)";
+            break;
+        }
+        case LANGUAGE_OSL_1:
+        {
+            kw << float4Keyword() << "(" 
+               << "(" << a << ".rgb.r > " << b << ".x) ? 1.0 : 0.0, "
+               << "(" << a << ".rgb.g > " << b << ".y) ? 1.0 : 0.0, "
+               << "(" << a << ".rgb.b > " << b << ".z) ? 1.0 : 0.0, "
+               << "(" << a << ".a > "     << b << ".w) ? 1.0 : 0.0)";
             break;
         }
 
@@ -851,6 +1155,85 @@ std::string GpuShaderText::float4GreaterThan(const std::string & a,
     return kw.str();
 }
 
+std::string GpuShaderText::float3GreaterThanEqual(const std::string& a,
+    const std::string& b) const
+{
+    std::ostringstream kw;
+    switch (m_lang)
+    {
+    case GPU_LANGUAGE_GLSL_1_2:
+    case GPU_LANGUAGE_GLSL_1_3:
+    case GPU_LANGUAGE_GLSL_4_0:
+    case GPU_LANGUAGE_GLSL_ES_1_0:
+    case GPU_LANGUAGE_GLSL_ES_3_0:
+    case GPU_LANGUAGE_CG:
+    {
+        kw << float3Keyword() << "(greaterThanEqual( " << a << ", " << b << "))";
+        break;
+    }
+    case LANGUAGE_OSL_1:
+    case GPU_LANGUAGE_MSL_2_0:
+    case GPU_LANGUAGE_HLSL_SM_5_0:
+    {
+        kw << float3Keyword() << "("
+            << "(" << a << "[0] >= " << b << "[0]) ? 1.0 : 0.0, "
+            << "(" << a << "[1] >= " << b << "[1]) ? 1.0 : 0.0, "
+            << "(" << a << "[2] >= " << b << "[2]) ? 1.0 : 0.0)";
+        break;
+    }
+
+    default:
+    {
+        throw Exception("Unknown GPU shader language.");
+    }
+    }
+    return kw.str();
+}
+
+std::string GpuShaderText::float4GreaterThanEqual(const std::string& a,
+    const std::string& b) const
+{
+    std::ostringstream kw;
+    switch (m_lang)
+    {
+    case GPU_LANGUAGE_GLSL_1_2:
+    case GPU_LANGUAGE_GLSL_1_3:
+    case GPU_LANGUAGE_GLSL_4_0:
+    case GPU_LANGUAGE_GLSL_ES_1_0:
+    case GPU_LANGUAGE_GLSL_ES_3_0:
+    case GPU_LANGUAGE_CG:
+    {
+        kw << float4Keyword() << "(greaterThanEqual( " << a << ", " << b << "))";
+        break;
+    }
+    case GPU_LANGUAGE_MSL_2_0:
+    case GPU_LANGUAGE_HLSL_SM_5_0:
+    {
+        kw << float4Keyword() << "("
+            << "(" << a << "[0] >= " << b << "[0]) ? 1.0 : 0.0, "
+            << "(" << a << "[1] >= " << b << "[1]) ? 1.0 : 0.0, "
+            << "(" << a << "[2] >= " << b << "[2]) ? 1.0 : 0.0, "
+            << "(" << a << "[3] >= " << b << "[3]) ? 1.0 : 0.0)";
+        break;
+    }
+    case LANGUAGE_OSL_1:
+    {
+        kw << float4Keyword() << "("
+            << "(" << a << ".rgb.r >= " << b << ".x) ? 1.0 : 0.0, "
+            << "(" << a << ".rgb.g >= " << b << ".y) ? 1.0 : 0.0, "
+            << "(" << a << ".rgb.b >= " << b << ".z) ? 1.0 : 0.0, "
+            << "(" << a << ".a >= " << b << ".w) ? 1.0 : 0.0)";
+        break;
+    }
+
+    default:
+    {
+        throw Exception("Unknown GPU shader language.");
+    }
+    }
+    return kw.str();
+}
+
 std::string GpuShaderText::atan2(const std::string & y,
                                  const std::string & x) const
 {
@@ -861,15 +1244,25 @@ std::string GpuShaderText::atan2(const std::string & y,
         case GPU_LANGUAGE_GLSL_1_2:
         case GPU_LANGUAGE_GLSL_1_3:
         case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
         {
             // note: "atan" not "atan2"
             kw << "atan(" << y << ", " << x << ")";
             break;
         }
-        case GPU_LANGUAGE_HLSL_DX11:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
         {
-            // note: operand order is swapped in HLSL
-            kw << "atan2(" << x << ", " << y << ")";
+            // note: Various internet sources claim that the x & y arguments need to be
+            // swapped for HLSL (relative to GLSL).  However, recent testing on Windows
+            // has revealed that the argument order needs to be the same as GLSL.
+            kw << "atan2(" << y << ", " << x << ")";
+            break;
+        }
+        case LANGUAGE_OSL_1:
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            kw << "atan2(" << y << ", " << x << ")";
             break;
         }
 
@@ -881,6 +1274,38 @@ std::string GpuShaderText::atan2(const std::string & y,
     return kw.str();
 }
 
+std::string GpuShaderText::sign(const std::string & v) const
+{
+    std::ostringstream kw;
+    switch(m_lang)
+    {
+        case GPU_LANGUAGE_CG:
+        case GPU_LANGUAGE_GLSL_1_2:
+        case GPU_LANGUAGE_GLSL_1_3:
+        case GPU_LANGUAGE_GLSL_4_0:
+        case GPU_LANGUAGE_GLSL_ES_1_0:
+        case GPU_LANGUAGE_GLSL_ES_3_0:
+        case GPU_LANGUAGE_HLSL_SM_5_0:
+        case GPU_LANGUAGE_MSL_2_0:
+        {
+            kw << "sign(" << v << ");";
+            break;
+        }
+        case LANGUAGE_OSL_1:
+        {
+            kw << "sign(" << float4Const(v + ".rgb.r", v + ".rgb.g",
+                                         v + ".rgb.b", v + ".a") << ");";
+            break;
+        }
+        default:
+        {
+            throw Exception("Unknown GPU shader language.");
+        }
+    }
+    return kw.str();
+}
+
+
 std::string BuildResourceName(GpuShaderCreatorRcPtr & shaderCreator, const std::string & prefix,
                               const std::string & base)
 {
@@ -900,42 +1325,45 @@ std::string BuildResourceName(GpuShaderCreatorRcPtr & shaderCreator, const std::
 // with 0 being 18% grey.  Above about -5, it is pretty much exactly F-Stops but below
 // that it is a pseudo-log so that 0.0 is at -7 stops rather than -Inf (as in a pure log).
 //
-void AddLinToLogShader(GpuShaderText & st)
+void AddLinToLogShader(GpuShaderCreatorRcPtr & shaderCreator, GpuShaderText & st)
 {
+    const std::string pix(shaderCreator->getPixelName());
+
     st.newLine() << "{";   // establish scope so local variable names won't conflict
     st.indent();
-    st.newLine() << "const float xbrk = 0.0041318374739483946;";
-    st.newLine() << "const float shift = -0.000157849851665374;";
-    st.newLine() << "const float m = 1. / (0.18 + shift);";
-    st.newLine() << "const float base2 = 1.4426950408889634;";  // 1/log(2)
-    st.newLine() << "const float gain = 363.034608563;";
-    st.newLine() << "const float offs = -7.;";
-    st.newLine() << st.float3Decl("ylin") << " = outColor.rgb * gain + offs;";
-    st.newLine() << st.float3Decl("ylog") << " = base2 * log( ( outColor.rgb + shift ) * m );";
-    st.newLine() << "outColor.r = (outColor.r < xbrk) ? ylin.r : ylog.r;";
-    st.newLine() << "outColor.g = (outColor.g < xbrk) ? ylin.g : ylog.g;";
-    st.newLine() << "outColor.b = (outColor.b < xbrk) ? ylin.b : ylog.b;";
+    st.newLine() << st.floatKeywordConst() << " xbrk = 0.0041318374739483946;";
+    st.newLine() << st.floatKeywordConst() << " shift = -0.000157849851665374;";
+    st.newLine() << st.floatKeywordConst() << " m = 1. / (0.18 + shift);";
+    st.newLine() << st.floatKeywordConst() << " base2 = 1.4426950408889634;";  // 1/log(2)
+    st.newLine() << st.floatKeywordConst() << " gain = 363.034608563;";
+    st.newLine() << st.floatKeywordConst() << " offs = -7.;";
+    st.newLine() << st.float3Decl("ylin") << " = " << pix << ".rgb * gain + offs;";
+    st.newLine() << st.float3Decl("ylog") << " = base2 * log( ( " << pix << ".rgb + shift ) * m );";
+    st.newLine() << pix << ".rgb.r = (" << pix << ".rgb.r < xbrk) ? ylin.x : ylog.x;";
+    st.newLine() << pix << ".rgb.g = (" << pix << ".rgb.g < xbrk) ? ylin.y : ylog.y;";
+    st.newLine() << pix << ".rgb.b = (" << pix << ".rgb.b < xbrk) ? ylin.z : ylog.z;";
     st.dedent();
     st.newLine() << "}";
 }
 
-void AddLogToLinShader(GpuShaderText & st)
+void AddLogToLinShader(GpuShaderCreatorRcPtr & shaderCreator, GpuShaderText & st)
 {
+    const std::string pix(shaderCreator->getPixelName());
+
     st.newLine() << "{";   // establish scope so local variable names won't conflict
     st.indent();
-    st.newLine() << "const float ybrk = -5.5;";
-    st.newLine() << "const float shift = -0.000157849851665374;";
-    st.newLine() << "const float gain = 363.034608563;";
-    st.newLine() << "const float offs = -7.;";
-    st.newLine() << st.float3Decl("xlin") << " = (outColor.rgb - offs) / gain;";
+    st.newLine() << st.floatKeywordConst() << " ybrk = -5.5;";
+    st.newLine() << st.floatKeywordConst() << " shift = -0.000157849851665374;";
+    st.newLine() << st.floatKeywordConst() << " gain = 363.034608563;";
+    st.newLine() << st.floatKeywordConst() << " offs = -7.;";
+    st.newLine() << st.float3Decl("xlin") << " = (" << pix << ".rgb - offs) / gain;";
     st.newLine() << st.float3Decl("xlog") << " = pow( " << st.float3Const(2.0f)
-                                          <<        ", outColor.rgb ) * (0.18 + shift) - shift;";
-    st.newLine() << "outColor.r = (outColor.r < ybrk) ? xlin.r : xlog.r;";
-    st.newLine() << "outColor.g = (outColor.g < ybrk) ? xlin.g : xlog.g;";
-    st.newLine() << "outColor.b = (outColor.b < ybrk) ? xlin.b : xlog.b;";
+                                          <<          ", " << pix << ".rgb ) * (0.18 + shift) - shift;";
+    st.newLine() << pix << ".rgb.r = (" << pix << ".rgb.r < ybrk) ? xlin.x : xlog.x;";
+    st.newLine() << pix << ".rgb.g = (" << pix << ".rgb.g < ybrk) ? xlin.y : xlog.y;";
+    st.newLine() << pix << ".rgb.b = (" << pix << ".rgb.b < ybrk) ? xlin.z : xlog.z;";
     st.dedent();
     st.newLine() << "}";
 }
 
 } // namespace OCIO_NAMESPACE
-
