@@ -1,12 +1,15 @@
 // SPDX-License-Identifier: BSD-3-Clause
 // Copyright Contributors to the OpenColorIO Project.
 
-
 #include <algorithm>
 #include <fstream>
 #include <map>
 #include <sstream>
 #include <string.h>
+#include <iostream>
+#include <iterator>
+
+#include <pystring.h>
 
 #include <OpenColorIO/OpenColorIO.h>
 
@@ -14,13 +17,12 @@
 #include "FileTransform.h"
 #include "Logging.h"
 #include "Mutex.h"
+#include "OCIOZArchive.h"
 #include "ops/noop/NoOps.h"
 #include "PathUtils.h"
 #include "Platform.h"
-#include "pystring/pystring.h"
 #include "utils/StringUtils.h"
 
-
 namespace OCIO_NAMESPACE
 {
 FileTransformRcPtr FileTransform::Create()
@@ -156,6 +158,11 @@ const char * FileTransform::GetFormatExtensionByIndex(int index)
     return FormatRegistry::GetInstance().getFormatExtensionByIndex(FORMAT_CAPABILITY_READ, index);
 }
 
+bool FileTransform::IsFormatExtensionSupported(const char * extension)
+{
+    return FormatRegistry::GetInstance().isFormatExtensionSupported(extension);
+}
+
 std::ostream& operator<< (std::ostream& os, const FileTransform& t)
 {
     os << "<FileTransform ";
@@ -177,6 +184,45 @@ std::ostream& operator<< (std::ostream& os, const FileTransform& t)
     return os;
 }
 
+// Wrapper around ConfigIOProxy getLutData implementation.
+std::unique_ptr<std::istream> getLutData(
+    const Config & config, 
+    const std::string & filepath, 
+    std::ios_base::openmode mode)
+{
+    if (config.getConfigIOProxy())
+    {
+        std::vector<uint8_t> buffer = config.getConfigIOProxy()->getLutData(filepath.c_str());
+        std::stringstream ss;
+        ss.write(reinterpret_cast<const char*>(buffer.data()), buffer.size());
+
+        return std::unique_ptr<std::stringstream>(
+            new std::stringstream(std::move(ss))
+        );
+    }
+
+    // Default behavior. Return file stream.
+    return std::unique_ptr<std::ifstream>(
+        new std::ifstream(Platform::filenameToUTF(filepath).c_str(), mode)
+    );
+}
+
+// Close stream returned by getLutData
+void closeLutStream(const Config & config, const std::istream & istream)
+{
+    // No-op when it is using ConfigIOProxy since getLutData returns a vector<uint8_t>.
+    if (config.getConfigIOProxy() == nullptr)
+    {
+        // The std::istream is coming from a std::ifstream.
+        // Pointer cast to ifstream and then close it.
+        std::ifstream * pIfStream = (std::ifstream *) &istream;
+        if (pIfStream->is_open())
+        {
+            pIfStream->close();
+        }
+    }
+}
+
 bool CollectContextVariables(const Config &, 
                              const Context & context,
                              const FileTransform & tr,
@@ -193,6 +239,7 @@ bool CollectContextVariables(const Config &,
     ContextRcPtr ctxFilename = Context::Create();
     ctxFilename->setSearchPath(context.getSearchPath());
     ctxFilename->setWorkingDir(context.getWorkingDir());
+    ctxFilename->setConfigIOProxy(context.getConfigIOProxy());
 
     const std::string resolvedString = context.resolveStringVar(src, ctxFilename);
     if (0 != strcmp(resolvedString.c_str(), src))
@@ -210,18 +257,19 @@ bool CollectContextVariables(const Config &,
     ContextRcPtr emptyContext = Context::Create();
     emptyContext->setSearchPath(context.getSearchPath());
     emptyContext->setWorkingDir(context.getWorkingDir());
+    emptyContext->setConfigIOProxy(context.getConfigIOProxy());
 
     // Used to collect the context variables needed to resolve the search_path. Note that this may
     // contain some variables that are not actually used.
     ContextRcPtr ctxFilepath = Context::Create();
     ctxFilepath->setSearchPath(context.getSearchPath());
     ctxFilepath->setWorkingDir(context.getWorkingDir());
+    ctxFilepath->setConfigIOProxy(context.getConfigIOProxy());
 
     try
     {
         // TODO: resolveFileLocation() tests the file existence (i.e. uses FileExists()) which is
         // useless here, and it could potentially add some performance penalty.
-
         const std::string resolvedFilename
             = context.resolveFileLocation(resolvedString.c_str(), ctxFilepath);
 
@@ -240,6 +288,16 @@ bool CollectContextVariables(const Config &,
         usedContextVars->addStringVars(ctxFilepath);
     }
 
+    // Check if the CCCID is using a context variable and add it to the context if that's the case.
+    ContextRcPtr ctxCCCID = Context::Create();
+    const char * cccid = tr.getCCCId();
+    std::string resolvedCCCID = context.resolveStringVar(cccid, ctxCCCID);
+    if (0 != strcmp(resolvedCCCID.c_str(), cccid))
+    {
+        foundContextVars = true;
+        usedContextVars->addStringVars(ctxCCCID);
+    }
+
     return foundContextVars;
 }
 
@@ -469,6 +527,32 @@ const char * FormatRegistry::getFormatExtensionByIndex(int capability, int index
     return "";
 }
 
+bool FormatRegistry::isFormatExtensionSupported(const char * extension) const
+{
+    // Early return false with an input of just the dot or invalid pointer.
+    if (!extension || !*extension || 0 == strcmp(extension, "."))
+    {
+        return false;
+    }
+
+    // If dot is present at the start, pointer arithmetic increment up by one to ignore that dot.
+    FileFormatVectorMap::const_iterator iter;
+    if (extension[0] == '.')
+    {
+        iter = m_formatsByExtension.find(StringUtils::Lower(extension + 1));
+    }
+    else
+    {
+        iter = m_formatsByExtension.find(StringUtils::Lower(extension));
+    }
+
+    if (iter != m_formatsByExtension.end())
+    {
+        return true;
+    }
+    return false;
+}
+
 ///////////////////////////////////////////////////////////////////////////
 
 FileFormat::~FileFormat()
@@ -513,7 +597,8 @@ namespace
 void LoadFileUncached(FileFormat * & returnFormat,
                       CachedFileRcPtr & returnCachedFile,
                       const std::string & filepath,
-                      Interpolation interp)
+                      Interpolation interp,
+                      const Config& config)
 {
     returnFormat = NULL;
 
@@ -542,14 +627,16 @@ void LoadFileUncached(FileFormat * & returnFormat,
     {
 
         FileFormat * tryFormat = *itFormat;
-        std::ifstream filestream;
+        std::unique_ptr<std::istream> pStream = nullptr;
         try
         {
-            // Open the filePath
-            filestream.open(
-                filepath.c_str(),
-                tryFormat->isBinary()
-                    ? std::ios_base::binary : std::ios_base::in);
+            pStream = getLutData(
+                config,
+                filepath, 
+                tryFormat->isBinary() ? std::ios_base::binary : std::ios_base::in 
+            );
+            auto & filestream = *pStream;
+            
             if (!filestream.good())
             {
                 std::ostringstream os;
@@ -572,14 +659,16 @@ void LoadFileUncached(FileFormat * & returnFormat,
 
             returnFormat = tryFormat;
             returnCachedFile = cachedFile;
-            filestream.close();
+
+            closeLutStream(config, filestream);
+
             return;
         }
         catch(std::exception & e)
         {
-            if (filestream.is_open())
+            if (pStream)
             {
-                filestream.close();
+                closeLutStream(config, *pStream); 
             }
 
             primaryErrorText += "    '";
@@ -615,11 +704,16 @@ void LoadFileUncached(FileFormat * & returnFormat,
         if(itAlt != endFormat)
             continue;
 
-        std::ifstream filestream;
+        std::unique_ptr<std::istream> pStream = nullptr;
         try
         {
-            filestream.open(filepath.c_str(), altFormat->isBinary()
-                ? std::ios_base::binary : std::ios_base::in);
+            pStream = getLutData(
+                config,
+                filepath, 
+                altFormat->isBinary() ? std::ios_base::binary : std::ios_base::in 
+            );
+            auto& filestream = *pStream;
+            
             if (!filestream.good())
             {
                 std::ostringstream os;
@@ -643,16 +737,18 @@ void LoadFileUncached(FileFormat * & returnFormat,
 
             returnFormat = altFormat;
             returnCachedFile = cachedFile;
-            filestream.close();
+
+            closeLutStream(config, filestream);
+
             return;
         }
         catch(std::exception & e)
         {
-            if (filestream.is_open())
+            if (pStream)
             {
-                filestream.close();
+                closeLutStream(config, *pStream); 
             }
-
+            
             if(IsDebugLoggingEnabled())
             {
                 std::ostringstream os;
@@ -720,11 +816,11 @@ typedef OCIO_SHARED_PTR<FileCacheResult> FileCacheResultPtr;
 template class GenericCache<std::string, FileCacheResultPtr>;
 GenericCache<std::string, FileCacheResultPtr> g_fileCache;
 
-
 void GetCachedFileAndFormat(FileFormat * & format,
                             CachedFileRcPtr & cachedFile,
                             const std::string & filepath,
-                            Interpolation interp)
+                            Interpolation interp,
+                            const Config& config)
 {
     // Have a two-mutex approach to decouple the cache entry creation from
     // the data creation. It was originally done to improve the multi-threaded
@@ -763,7 +859,7 @@ void GetCachedFileAndFormat(FileFormat * & format,
 
         try
         {
-            LoadFileUncached(result->format, result->cachedFile, filepath, interp);
+            LoadFileUncached(result->format, result->cachedFile, filepath, interp, config);
         }
         catch (std::exception & e)
         {
@@ -855,7 +951,11 @@ void BuildFileTransformOps(OpRcPtrVec & ops,
     FileFormat* format = NULL;
     CachedFileRcPtr cachedFile;
 
-    GetCachedFileAndFormat(format, cachedFile, filepath, fileTransform.getInterpolation());
+    GetCachedFileAndFormat(format, 
+                           cachedFile, 
+                           filepath, 
+                           fileTransform.getInterpolation(),
+                           config);
 
     try
     {
@@ -886,4 +986,5 @@ void BuildFileTransformOps(OpRcPtrVec & ops,
         throw Exception(err.str().c_str());
     }
 }
+
 } // namespace OCIO_NAMESPACE
