@@ -11,48 +11,36 @@
 #include <OpenColorIO/OpenColorIO.h>
 namespace OCIO = OCIO_NAMESPACE;
 
-#include <OpenImageIO/imageio.h>
-#include <OpenImageIO/typedesc.h>
-#if (OIIO_VERSION < 10100)
-namespace OIIO = OIIO_NAMESPACE;
-#endif
-
 #include "apputils/argparse.h"
 
 #ifdef OCIO_GPU_ENABLED
 #include "oglapp.h"
 #endif // OCIO_GPU_ENABLED
 
-#include "oiiohelpers.h"
-#include "OpenEXR/half.h"
-
+#include "imageio.h"
 
 // Array of non OpenColorIO arguments.
 static std::vector<std::string> args;
 
 // Fill 'args' array with OpenColorIO arguments.
-static int
-parse_end_args(int argc, const char *argv[])
+static int parse_end_args(int argc, const char *argv[])
 {
-  while(argc>0)
-  {
-    args.push_back(argv[0]);
-    argc--;
-    argv++;
-  }
+    while (argc>0)
+    {
+        args.push_back(argv[0]);
+        argc--;
+        argv++;
+    }
 
-  return 0;
+    return 0;
 }
 
-bool ParseNameValuePair(std::string& name, std::string& value,
-                        const std::string& input);
+bool ParseNameValuePair(std::string& name, std::string& value, const std::string& input);
 
 bool StringToFloat(float * fval, const char * str);
 
 bool StringToInt(int * ival, const char * str);
 
-bool StringToVector(std::vector<int> * ivector, const char * str);
-
 int main(int argc, const char **argv)
 {
     ArgParse ap;
@@ -60,42 +48,51 @@ int main(int argc, const char **argv)
     std::vector<std::string> floatAttrs;
     std::vector<std::string> intAttrs;
     std::vector<std::string> stringAttrs;
-    std::string keepChannels;
 
-    bool croptofull     = false;
-    bool usegpu         = false;
-    bool usegpuLegacy   = false;
-    bool outputgpuInfo  = false;
-    bool verbose        = false;
-    bool help           = false;
-    bool useLut         = false;
-    bool useDisplayView = false;
+    std::string outputDepth;
+
+    bool usegpu                 = false;
+    bool usegpuLegacy           = false;
+    bool outputgpuInfo          = false;
+    bool verbose                = false;
+    bool help                   = false;
+    bool useLut                 = false;
+    bool useDisplayView         = false;
+    bool useInvertView          = false;
+    bool useNamedTransform      = false;
+    bool useInvNamedTransform   = false;
 
     ap.options("ocioconvert -- apply colorspace transform to an image \n\n"
-               "usage: ocioconvert [options]  inputimage inputcolorspace outputimage outputcolorspace\n"
+               "usage: ocioconvert [options] inputimage inputcolorspace outputimage outputcolorspace\n"
                "   or: ocioconvert [options] --lut lutfile inputimage outputimage\n"
-               "   or: ocioconvert [options] --view inputimage inputcolorspace outputimage displayname viewname\n\n",
+               "   or: ocioconvert [options] --view inputimage inputcolorspace outputimage displayname viewname\n"
+               "   or: ocioconvert [options] --invertview inputimage displayname viewname outputimage outputcolorspace\n"
+               "   or: ocioconvert [options] --namedtransform transformname inputimage outputimage\n"
+               "   or: ocioconvert [options] --invnamedtransform transformname inputimage outputimage\n\n",
                "%*", parse_end_args, "",
                "<SEPARATOR>", "Options:",
-               "--lut",       &useLut,         "Convert using a LUT rather than a config file",
-               "--view",      &useDisplayView, "Convert to a (display,view) pair rather than to "
-                                               "an output color space",
-               "--gpu",       &usegpu,         "Use GPU color processing instead of CPU (CPU is the default)",
-               "--gpulegacy", &usegpuLegacy,   "Use the legacy (i.e. baked) GPU color processing "
-                                               "instead of the CPU one (--gpu is ignored)",
-               "--gpuinfo",  &outputgpuInfo,   "Output the OCIO shader program",
-               "--help",     &help,            "Print help message",
-               "-v" ,        &verbose,         "Display general information",
-               "<SEPARATOR>", "\nOpenImageIO options:",
+               "--lut",                 &useLut,                "Convert using a LUT rather than a config file",
+               "--view",                &useDisplayView,        "Convert to a (display,view) pair rather than to "
+                                                                "an output color space",
+               "--invertview",          &useInvertView,         "Convert from a (display,view) pair rather than "
+                                                                "from a color space",
+               "--namedtransform",      &useNamedTransform,     "Convert using a named transform in the forward direction",
+               "--invnamedtransform",   &useInvNamedTransform,  "Convert using a named transform in the inverse direction",
+               "--gpu",                 &usegpu,                "Use GPU color processing instead of CPU (CPU is the default)",
+               "--gpulegacy",           &usegpuLegacy,          "Use the legacy (i.e. baked) GPU color processing "
+                                                                "instead of the CPU one (--gpu is ignored)",
+               "--gpuinfo",             &outputgpuInfo,         "Output the OCIO shader program",
+               "--h",                   &help,                  "Display the help and exit",
+               "--help",                &help,                  "Display the help and exit",
+               "-v" ,                   &verbose,               "Display general information",
+               "<SEPARATOR>", "\nOpenImageIO or OpenEXR options:",
+               "--bitdepth %s",         &outputDepth,  "Output image bitdepth",
                "--float-attribute %L",  &floatAttrs,   "\"name=float\" pair defining OIIO float attribute "
                                                        "for outputimage",
-               "--int-attribute %L",    &intAttrs,     "\"name=int\" pair defining OIIO int attribute "
+               "--int-attribute %L",    &intAttrs,     "\"name=int\" pair defining an int attribute "
                                                        "for outputimage",
-               "--string-attribute %L", &stringAttrs,  "\"name=string\" pair defining OIIO string attribute "
+               "--string-attribute %L", &stringAttrs,  "\"name=string\" pair defining a string attribute "
                                                        "for outputimage",
-               "--croptofull",          &croptofull,   "Crop or pad to make pixel data region match the "
-                                                       "\"full\" region",
-               "--ch %s",               &keepChannels, "Select channels (e.g., \"2,3,4\")",
                NULL
                );
 
@@ -121,15 +118,41 @@ int main(int argc, const char **argv)
     }
 #endif // OCIO_GPU_ENABLED
 
-    const char * inputimage       = nullptr;
-    const char * inputcolorspace  = nullptr;
-    const char * outputimage      = nullptr;
-    const char * outputcolorspace = nullptr;
-    const char * lutFile          = nullptr;
-    const char * display          = nullptr;
-    const char * view             = nullptr;
+    OCIO::BitDepth userOutputBitDepth = OCIO::BIT_DEPTH_UNKNOWN;
+    if (!outputDepth.empty())
+    {
+        if (outputDepth == "uint8")
+        {
+            userOutputBitDepth = OCIO::BIT_DEPTH_UINT8;
+        }
+        else if (outputDepth == "uint16")
+        {
+            userOutputBitDepth = OCIO::BIT_DEPTH_UINT16;
+        }
+        else if (outputDepth == "half")
+        {
+            userOutputBitDepth = OCIO::BIT_DEPTH_F16;
+        }
+        else if (outputDepth == "float")
+        {
+            userOutputBitDepth = OCIO::BIT_DEPTH_F32;
+        }
+        else
+        {
+            throw OCIO::Exception("Unsupported output bitdepth, must be uint8, uint16, half or float.");
+        }
+    }
 
-    if (!useLut && !useDisplayView)
+    const char * inputimage         = nullptr;
+    const char * inputcolorspace    = nullptr;
+    const char * outputimage        = nullptr;
+    const char * outputcolorspace   = nullptr;
+    const char * lutFile            = nullptr;
+    const char * display            = nullptr;
+    const char * view               = nullptr;
+    const char * namedtransform     = nullptr;
+ 
+    if (!useLut && !useDisplayView && !useInvertView && !useNamedTransform && !useInvNamedTransform)
     {
         if (args.size() != 4)
         {
@@ -177,14 +200,79 @@ int main(int argc, const char **argv)
         display         = args[3].c_str();
         view            = args[4].c_str();
     }
+    else if (useDisplayView && useInvertView)
+    {
+        std::cerr << "ERROR: Options view & invertview can't be used at the same time." << std::endl;
+        ap.usage();
+        exit(1);
+    }
+    else if (useInvertView) 
+    {
+        if (args.size() != 5)
+        {
+            std::cerr << "ERROR: Expecting 5 arguments for --invertview option, found "
+                      << args.size() << "." << std::endl;
+            ap.usage();
+            exit(1);
+        }
+        inputimage          = args[0].c_str();
+        display             = args[1].c_str();
+        view                = args[2].c_str();
+        outputimage         = args[3].c_str();
+        outputcolorspace    = args[4].c_str();
+    }
+    else if (useNamedTransform)
+    {
+        if (useLut || useDisplayView || useInvertView || useInvNamedTransform)
+        {
+            std::cerr << "ERROR: Option namedtransform can't be used with lut, view, invertview, \
+                or invnamedtransform at the same time." << std::endl;
+            ap.usage();
+            exit(1);
+        }
+        
+        if (args.size() != 3)
+        {
+            std::cerr << "ERROR: Expecting 3 arguments for --namedtransform option, found "
+                << args.size() << "." << std::endl;
+            ap.usage();
+            exit(1);
+        }
+        
+        namedtransform  = args[0].c_str();
+        inputimage      = args[1].c_str();
+        outputimage     = args[2].c_str();
+    }
+    else if (useInvNamedTransform)
+    {
+        if (useLut || useDisplayView || useInvertView || useNamedTransform)
+        {
+            std::cerr << "ERROR: Option invnamedtransform can't be used with lut, view, invertview, \
+                or namedtransform at the same time." << std::endl;
+            ap.usage();
+            exit(1);
+        }
 
-    if(verbose)
+        if (args.size() != 3)
+        {
+            std::cerr << "ERROR: Expecting 3 arguments for --invnamedtransform option, found "
+                << args.size() << "." << std::endl;
+            ap.usage();
+            exit(1);
+        }
+
+        namedtransform  = args[0].c_str();
+        inputimage      = args[1].c_str();
+        outputimage     = args[2].c_str();
+    }
+
+    if (verbose)
     {
         std::cout << std::endl;
-        std::cout << "OIIO Version: " << OIIO_VERSION_STRING << std::endl;
+        std::cout << OCIO::ImageIO::GetVersion() << std::endl;
         std::cout << "OCIO Version: " << OCIO::GetVersion() << std::endl;
         const char * env = OCIO::GetEnvVariable("OCIO");
-        if(env && *env && !useLut)
+        if (env && *env && !useLut)
         {
             try
             {
@@ -200,10 +288,10 @@ int main(int argc, const char **argv)
                 std::cout << "ERROR loading config file: " << e.what() << std::endl;
                 exit(1);
             }
-            catch(...)
+            catch (...)
             {
 
-                std::cerr << "ERROR loading the config file: '" << env << "'";
+                std::cerr << "ERROR loading config file: '" << env << "'" << std::endl;
                 exit(1);
             }
         }
@@ -220,181 +308,33 @@ int main(int argc, const char **argv)
         std::cout << "Using GPU color processing." << std::endl;
     }
 
-    OIIO::ImageSpec spec;
-    OCIO::ImgBuffer img;
-    int imgwidth = 0;
-    int imgheight = 0;
-    int components = 0;
+    OCIO::ImageIO imgInput;
+    OCIO::ImageIO imgOutputCPU;
+    // Default is to perform in-place conversion.
+    OCIO::ImageIO *imgOutput = &imgInput;
 
     // Load the image.
     std::cout << std::endl;
     std::cout << "Loading " << inputimage << std::endl;
     try
     {
-#if OIIO_VERSION < 10903
-        OIIO::ImageInput* f = OIIO::ImageInput::create(inputimage);
-#else
-        auto f = OIIO::ImageInput::create(inputimage);
-#endif
-        if(!f)
-        {
-            std::cerr << "ERROR: Could not create image input." << std::endl;
-            exit(1);
-        }
-
-        f->open(inputimage, spec);
-
-        std::string error = f->geterror();
-        if(!error.empty())
-        {
-            std::cerr << "ERROR: Could not load image: " << error << std::endl;
-            exit(1);
-        }
-
-        OCIO::PrintImageSpec(spec, verbose);
-
-        imgwidth = spec.width;
-        imgheight = spec.height;
-        components = spec.nchannels;
-
         if (usegpu || usegpuLegacy)
         {
-            spec.format = OIIO::TypeDesc::FLOAT;
-            img.allocate(spec);
-
-            const bool ok = f->read_image(spec.format, img.getBuffer());
-            if(!ok)
-            {
-                std::cerr << "ERROR: Reading \"" << inputimage << "\" failed with: "
-                          << f->geterror() << "." << std::endl;
-                exit(1);
-            }
-
-            if(croptofull)
-            {
-                std::cerr << "ERROR: Crop disabled in GPU mode." << std::endl;
-                exit(1);
-            }
+            imgInput.read(inputimage, OCIO::BIT_DEPTH_F32);
         }
         else
         {
-            img.allocate(spec);
-
-            const bool ok = f->read_image(spec.format, img.getBuffer());
-            if(!ok)
-            {
-                std::cerr << "ERROR: Reading \"" << inputimage << "\" failed with: "
-                          << f->geterror() << "." << std::endl;
-                exit(1);
-            }
+            imgInput.read(inputimage);
         }
 
-#if OIIO_VERSION < 10903
-        OIIO::ImageInput::destroy(f);
-#endif
-
-        std::vector<int> kchannels;
-        //parse --ch argument.
-        if (keepChannels != "" && !StringToVector(&kchannels, keepChannels.c_str()))
-        {
-            std::cerr << "ERROR: --ch: '" << keepChannels
-                      << "' should be comma-seperated integers." << std::endl;
-            exit(1);
-        }
-
-        //if kchannels not specified, then keep all channels.
-        if (kchannels.size() == 0)
-        {
-            kchannels.resize(components);
-            for (int channel=0; channel < components; channel++)
-            {
-                kchannels[channel] = channel;
-            }
-        }
-
-        if (croptofull)
-        {
-            imgwidth = spec.full_width;
-            imgheight = spec.full_height;
-
-            std::cout << "cropping to " << imgwidth
-                      << "x" << imgheight << std::endl;
-        }
-
-        if (croptofull || (int)kchannels.size() < spec.nchannels)
-        {
-            // Redefine the spec so it matches the new bounding box.
-            OIIO::ImageSpec croppedSpec = spec;
-
-            croppedSpec.x = 0;
-            croppedSpec.y = 0;
-            croppedSpec.height    = imgheight;
-            croppedSpec.width     = imgwidth;
-            croppedSpec.nchannels = (int)(kchannels.size());
-
-            OCIO::ImgBuffer croppedImg(croppedSpec);
-
-            void * croppedBuf = croppedImg.getBuffer();
-            void * imgBuf     = img.getBuffer();
-
-            // crop down bounding box and ditch all but n channels.
-            // img is a flattened 3 dimensional matrix heightxwidthxchannels.
-            // fill croppedimg with only the needed pixels.
-            for (int y=0 ; y < spec.height ; y++)
-            {
-                for (int x=0 ; x < spec.width; x++)
-                {
-                    for (int k=0; k < (int)kchannels.size(); k++)
-                    {
-                        int channel = kchannels[k];
-                        int current_pixel_y = y + spec.y;
-                        int current_pixel_x = x + spec.x;
-
-                        if (current_pixel_y >= 0 &&
-                            current_pixel_x >= 0 &&
-                            current_pixel_y < imgheight &&
-                            current_pixel_x < imgwidth)
-                        {
-                            const size_t imgIdx = (y * spec.width * components) 
-                                                    + (x * components) + channel;
-
-                            const size_t cropIdx = (current_pixel_y * imgwidth * kchannels.size())
-                                                    + (current_pixel_x * kchannels.size())
-                                                    + channel;
-
-                            if(spec.format==OIIO::TypeDesc::FLOAT)
-                            {
-                                ((float*)croppedBuf)[cropIdx] = ((float*)imgBuf)[imgIdx];
-                            }
-                            else if(spec.format==OIIO::TypeDesc::HALF)
-                            {
-                                ((half*)croppedBuf)[cropIdx] = ((half*)imgBuf)[imgIdx];
-                            }
-                            else if(spec.format==OIIO::TypeDesc::UINT16)
-                            {
-                                ((uint16_t*)croppedBuf)[cropIdx] = ((uint16_t*)imgBuf)[imgIdx];
-                            }
-                            else if(spec.format==OIIO::TypeDesc::UINT8)
-                            {
-                                ((uint8_t*)croppedBuf)[cropIdx] = ((uint8_t*)imgBuf)[imgIdx];
-                            }
-                            else
-                            {
-                                std::cerr << "ERROR: Unsupported image type: " 
-                                          << spec.format << "." << std::endl;
-                                exit(1);
-                            }
-                        }
-                    }
-                }
-            }
-
-            components = (int)(kchannels.size());
-
-            img = std::move(croppedImg);
-        }
+        std::cout << imgInput.getImageDescStr() << std::endl;
     }
-    catch(...)
+    catch (const std::exception & e)
+    {
+        std::cerr << "ERROR: Loading file failed: " << e.what() << std::endl;
+        exit(1);
+    }
+    catch (...)
     {
         std::cerr << "ERROR: Loading file failed." << std::endl;
         exit(1);
@@ -407,17 +347,18 @@ int main(int argc, const char **argv)
     if (usegpu || usegpuLegacy)
     {
         OCIO::OglApp::Components comp = OCIO::OglApp::COMPONENTS_RGBA;
-        if (components == 4)
+        if (imgInput.getNumChannels() == 4)
         {
             comp = OCIO::OglApp::COMPONENTS_RGBA;
         }
-        else if (components == 3)
+        else if (imgInput.getNumChannels() == 3)
         {
             comp = OCIO::OglApp::COMPONENTS_RGB;
         }
         else
         {
-            std::cerr << "Cannot convert image with " << components << " components." << std::endl;
+            std::cerr << "Cannot convert image with " << imgInput.getNumChannels()
+                      << " components." << std::endl;
             exit(1);
         }
 
@@ -438,7 +379,7 @@ int main(int argc, const char **argv)
 
         oglApp->setPrintShader(outputgpuInfo);
 
-        oglApp->initImage(imgwidth, imgheight, comp, (float *)img.getBuffer());
+        oglApp->initImage(imgInput.getWidth(), imgInput.getHeight(), comp, (float *)imgInput.getData());
         
         oglApp->createGLBuffers();
     }
@@ -473,6 +414,42 @@ int main(int argc, const char **argv)
                 t->setView(view);
                 processor = config->getProcessor(t);
             }
+            else if (useInvertView)
+            {
+                OCIO::DisplayViewTransformRcPtr t = OCIO::DisplayViewTransform::Create();
+                t->setSrc(outputcolorspace);
+                t->setDisplay(display);
+                t->setView(view);
+                processor = config->getProcessor(t, OCIO::TRANSFORM_DIR_INVERSE);
+            }
+            else if (useNamedTransform)
+            {
+                auto nt = config->getNamedTransform(namedtransform);
+
+                if (nt)
+                {
+                    processor = config->getProcessor(nt, OCIO::TRANSFORM_DIR_FORWARD);
+                }
+                else
+                {
+                   std::cout << "ERROR: Could not get NamedTransform " << namedtransform << std::endl;
+                   exit(1);
+                }                
+            }
+            else if (useInvNamedTransform)
+            {
+                auto nt = config->getNamedTransform(namedtransform);
+
+                if (nt)
+                {
+                    processor = config->getProcessor(nt, OCIO::TRANSFORM_DIR_INVERSE);
+                }
+                else
+                {
+                    std::cout << "ERROR: Could not get NamedTransform " << namedtransform << std::endl;
+                    exit(1);
+                }
+            }
             else
             {
                 processor = config->getProcessor(inputcolorspace, outputcolorspace);
@@ -502,26 +479,81 @@ int main(int argc, const char **argv)
             gpu->extractGpuShaderInfo(shaderDesc);
 
             oglApp->setShader(shaderDesc);
-            oglApp->reshape(imgwidth, imgheight);
+            oglApp->reshape(imgInput.getWidth(), imgInput.getHeight());
             oglApp->redisplay();
-            oglApp->readImage((float *)img.getBuffer());
+            oglApp->readImage((float *)imgInput.getData());
         }
         else
 #endif // OCIO_GPU_ENABLED
         {
-            const OCIO::BitDepth bitDepth = OCIO::GetBitDepth(spec);
+            /*
+                Set the bit-depth of the output buffer.
 
-            OCIO::ConstCPUProcessorRcPtr cpuProcessor 
-                = processor->getOptimizedCPUProcessor(bitDepth, bitDepth,
+                Whereas the GPU processor always work on float data, the CPU processor
+                can be optimised for a specific input and output bit-depth.
+
+                The converted image may require more bits than the source image.
+                For example, converting a log image to linear requires at least a half-float
+                output format. For most cases, half-float strikes a good balance between
+                precision and storage space. But if the input depth would lose precision
+                when converted to half-float, use float for the output depth instead.
+
+                Note that when using OpenImageIO, the actual output bit-depth may be overrided
+                if the file format doesn't support it. OCIO is not trying to analyze the filename
+                to emulate OpenImageIO's decision making process.
+            */
+            const OCIO::BitDepth inputBitDepth = imgInput.getBitDepth();
+            OCIO::BitDepth outputBitDepth;
+
+            if (userOutputBitDepth != OCIO::BIT_DEPTH_UNKNOWN)
+            {
+                outputBitDepth = userOutputBitDepth;
+            }
+            else
+            {
+                if (inputBitDepth == OCIO::BIT_DEPTH_UINT16 || inputBitDepth == OCIO::BIT_DEPTH_F32)
+                {
+                    outputBitDepth = OCIO::BIT_DEPTH_F32;
+                }
+                else if (inputBitDepth == OCIO::BIT_DEPTH_UINT8 || inputBitDepth == OCIO::BIT_DEPTH_F16)
+                {
+                    outputBitDepth = OCIO::BIT_DEPTH_F16;
+                }
+                else
+                {
+                    throw OCIO::Exception("Unsupported input bitdepth, must be uint8, uint16, half or float.");
+                }
+            }
+
+            OCIO::ConstCPUProcessorRcPtr cpuProcessor
+                = processor->getOptimizedCPUProcessor(inputBitDepth,
+                                                      outputBitDepth,
                                                       OCIO::OPTIMIZATION_DEFAULT);
 
+            const bool useOutputBuffer = inputBitDepth != outputBitDepth;
+
+            if (useOutputBuffer)
+            {
+                imgOutputCPU.init(imgInput, outputBitDepth);
+                imgOutput = &imgOutputCPU;
+            }
+
             const std::chrono::high_resolution_clock::time_point start
                 = std::chrono::high_resolution_clock::now();
 
-            OCIO::ImageDescRcPtr imgDesc = OCIO::CreateImageDesc(spec, img);
-            cpuProcessor->apply(*imgDesc);
+            if (useOutputBuffer)
+            {
+                OCIO::ImageDescRcPtr srcImgDesc = imgInput.getImageDesc();
+                OCIO::ImageDescRcPtr dstImgDesc = imgOutputCPU.getImageDesc();
+                cpuProcessor->apply(*srcImgDesc, *dstImgDesc);
+            }
+            else
+            {
+                OCIO::ImageDescRcPtr imgDesc = imgInput.getImageDesc();
+                cpuProcessor->apply(*imgDesc);
+            }
 
-            if(verbose)
+            if (verbose)
             {
                 const std::chrono::high_resolution_clock::time_point end
                     = std::chrono::high_resolution_clock::now();
@@ -535,69 +567,67 @@ int main(int argc, const char **argv)
             }
         }
     }
-    catch(OCIO::Exception & exception)
+    catch (const OCIO::Exception & exception)
     {
         std::cerr << "ERROR: OCIO failed with: " << exception.what() << std::endl;
         exit(1);
     }
-    catch(...)
+    catch (...)
     {
         std::cerr << "ERROR: Unknown error processing the image." << std::endl;
         exit(1);
     }
 
-    //
-    // set the provided OpenImageIO attributes.
-    //
-    bool parseerror = false;
-    for(unsigned int i=0; i<floatAttrs.size(); ++i)
+    // Set the provided image attributes.
+    bool parseError = false;
+    for (unsigned int i=0; i<floatAttrs.size(); ++i)
     {
         std::string name, value;
         float fval = 0.0f;
 
-        if(!ParseNameValuePair(name, value, floatAttrs[i]) ||
+        if (!ParseNameValuePair(name, value, floatAttrs[i]) ||
            !StringToFloat(&fval,value.c_str()))
         {
             std::cerr << "ERROR: Attribute string '" << floatAttrs[i]
                       << "' should be in the form name=floatvalue." << std::endl;
-            parseerror = true;
+            parseError = true;
             continue;
         }
 
-        spec.attribute(name, fval);
+        imgOutput->attribute(name, fval);
     }
 
-    for(unsigned int i=0; i<intAttrs.size(); ++i)
+    for (unsigned int i=0; i<intAttrs.size(); ++i)
     {
         std::string name, value;
         int ival = 0;
-        if(!ParseNameValuePair(name, value, intAttrs[i]) ||
+        if (!ParseNameValuePair(name, value, intAttrs[i]) ||
            !StringToInt(&ival,value.c_str()))
         {
             std::cerr << "ERROR: Attribute string '" << intAttrs[i]
                       << "' should be in the form name=intvalue." << std::endl;
-            parseerror = true;
+            parseError = true;
             continue;
         }
 
-        spec.attribute(name, ival);
+        imgOutput->attribute(name, ival);
     }
 
-    for(unsigned int i=0; i<stringAttrs.size(); ++i)
+    for (unsigned int i=0; i<stringAttrs.size(); ++i)
     {
         std::string name, value;
-        if(!ParseNameValuePair(name, value, stringAttrs[i]))
+        if (!ParseNameValuePair(name, value, stringAttrs[i]))
         {
             std::cerr << "ERROR: Attribute string '" << stringAttrs[i]
                       << "' should be in the form name=value." << std::endl;
-            parseerror = true;
+            parseError = true;
             continue;
         }
 
-        spec.attribute(name, value);
+        imgOutput->attribute(name, value);
     }
 
-    if(parseerror)
+    if (parseError)
     {
         exit(1);
     }
@@ -605,17 +635,6 @@ int main(int argc, const char **argv)
     // Write out the result.
     try
     {
-#if OIIO_VERSION < 10903
-        OIIO::ImageOutput* f = OIIO::ImageOutput::create(outputimage);
-#else
-        auto f = OIIO::ImageOutput::create(outputimage);
-#endif
-        if(!f)
-        {
-            std::cerr << "ERROR: Could not create output input." << std::endl;
-            exit(1);
-        }
-
         if (useDisplayView)
         {
             OCIO::ConstConfigRcPtr config = OCIO::GetCurrentConfig();
@@ -624,31 +643,19 @@ int main(int argc, const char **argv)
 
         if (outputcolorspace)
         {
-            spec.attribute("oiio:ColorSpace", outputcolorspace);
+            imgOutput->attribute("oiio:ColorSpace", outputcolorspace);
         }
 
-        f->open(outputimage, spec);
-
-        if(!f->write_image(spec.format, img.getBuffer()))
-        {
-            std::cerr << "ERROR: Writing \"" << outputimage << "\" failed with: "
-                      << f->geterror() << "." << std::endl;
-            exit(1);
-        }
-
-        f->close();
-#if OIIO_VERSION < 10903
-        OIIO::ImageOutput::destroy(f);
-#endif
+        imgOutput->write(outputimage, userOutputBitDepth);
     }
-    catch(...)
+    catch (...)
     {
         std::cerr << "ERROR: Writing file \"" << outputimage << "\"." << std::endl;
         exit(1);
     }
 
-    std::cout << std::endl;
     std::cout << "Wrote " << outputimage << std::endl;
+    std::cout << imgOutput->getImageDescStr() << std::endl;
 
     return 0;
 }
@@ -663,7 +670,10 @@ bool ParseNameValuePair(std::string& name,
 {
     // split string into name=value.
     size_t pos = input.find('=');
-    if(pos==std::string::npos) return false;
+    if (pos==std::string::npos)
+    {
+        return false;
+    }
 
     name = input.substr(0,pos);
     value = input.substr(pos+1);
@@ -673,49 +683,42 @@ bool ParseNameValuePair(std::string& name,
 // return true on success.
 bool StringToFloat(float * fval, const char * str)
 {
-    if(!str) return false;
-
-    std::istringstream inputStringstream(str);
-    float x;
-    if(!(inputStringstream >> x))
+    if (!str)
     {
         return false;
     }
 
-    if(fval) *fval = x;
+    std::istringstream inputStringstream(str);
+    float x;
+    if (!(inputStringstream >> x))
+    {
+        return false;
+    }
+
+    if (fval)
+    {
+        *fval = x;
+    }
     return true;
 }
 
 bool StringToInt(int * ival, const char * str)
 {
-    if(!str) return false;
-
-    std::istringstream inputStringstream(str);
-    int x;
-    if(!(inputStringstream >> x))
+    if (!str)
     {
         return false;
     }
 
-    if(ival) *ival = x;
+    std::istringstream inputStringstream(str);
+    int x;
+    if (!(inputStringstream >> x))
+    {
+        return false;
+    }
+
+    if (ival)
+    {
+        *ival = x;
+    }
     return true;
 }
-
-bool StringToVector(std::vector<int> * ivector, const char * str)
-{
-    std::stringstream ss(str);
-    int i;
-    while (ss >> i)
-    {
-        ivector->push_back(i);
-        if (ss.peek() == ',')
-        {
-          ss.ignore();
-        }
-    }
-    return ivector->size() != 0;
-}
-
-
-
-
