@@ -33,7 +33,10 @@ enum ConfigIterator
     IT_NAMED_TRANSFORM_NAME,
     IT_NAMED_TRANSFORM,
     IT_ACTIVE_NAMED_TRANSFORM_NAME,
-    IT_ACTIVE_NAMED_TRANSFORM
+    IT_ACTIVE_NAMED_TRANSFORM,
+    IT_DISPLAY_ALL,
+    IT_DISPLAY_VIEW_TYPE,
+    IT_VIRTUAL_DISPLAY_VIEW,
 };
 
 using EnvironmentVarNameIterator       = PyIterator<ConfigRcPtr, IT_ENVIRONMENT_VAR_NAME>;
@@ -45,16 +48,21 @@ using ColorSpaceNameIterator           = PyIterator<ConfigRcPtr,
 using ColorSpaceIterator               = PyIterator<ConfigRcPtr, 
                                                     IT_COLOR_SPACE, 
                                                     SearchReferenceSpaceType,
-                                                     ColorSpaceVisibility>;
+                                                    ColorSpaceVisibility>;
 using ActiveColorSpaceNameIterator     = PyIterator<ConfigRcPtr, IT_ACTIVE_COLOR_SPACE_NAME>;
 using ActiveColorSpaceIterator         = PyIterator<ConfigRcPtr, IT_ACTIVE_COLOR_SPACE>;
 using RoleNameIterator                 = PyIterator<ConfigRcPtr, IT_ROLE_NAME>;
 using RoleColorSpaceIterator           = PyIterator<ConfigRcPtr, IT_ROLE_COLOR_SPACE>;
 using DisplayIterator                  = PyIterator<ConfigRcPtr, IT_DISPLAY>;
+using DisplayAllIterator               = PyIterator<ConfigRcPtr, IT_DISPLAY_ALL>;
 using SharedViewIterator               = PyIterator<ConfigRcPtr, IT_SHARED_VIEW>;
+using VirtualViewIterator              = PyIterator<ConfigRcPtr, IT_VIRTUAL_DISPLAY_VIEW, 
+                                                    ViewType>;
 using ViewIterator                     = PyIterator<ConfigRcPtr, IT_DISPLAY_VIEW, std::string>;
 using ViewForColorSpaceIterator        = PyIterator<ConfigRcPtr, IT_DISPLAY_VIEW_COLORSPACE,
                                                     std::string, std::string>;
+using ViewForViewTypeIterator          = PyIterator<ConfigRcPtr, IT_DISPLAY_VIEW_TYPE, 
+                                                    ViewType, std::string>;
 using LookNameIterator                 = PyIterator<ConfigRcPtr, IT_LOOK_NAME>;
 using LookIterator                     = PyIterator<ConfigRcPtr, IT_LOOK>;
 using ViewTransformNameIterator        = PyIterator<ConfigRcPtr, IT_VIEW_TRANSFORM_NAME>;
@@ -110,10 +118,18 @@ void bindPyConfig(py::module & m)
         py::class_<DisplayIterator>(
             clsConfig, "DisplayIterator");
 
+    auto clsDisplayAllIterator = 
+        py::class_<DisplayAllIterator>(
+            clsConfig, "DisplayAllIterator");
+
     auto clsSharedViewIterator = 
         py::class_<SharedViewIterator>(
             clsConfig, "SharedViewIterator");
 
+    auto clsVirtualViewIterator = 
+        py::class_<VirtualViewIterator>(
+            clsConfig, "VirtualViewIterator");
+
     auto clsViewIterator = 
         py::class_<ViewIterator>(
             clsConfig, "ViewIterator");
@@ -122,6 +138,10 @@ void bindPyConfig(py::module & m)
         py::class_<ViewForColorSpaceIterator>(
             clsConfig, "ViewForColorSpaceIterator");
 
+    auto clsViewForViewTypeIterator =
+        py::class_<ViewForViewTypeIterator>(
+            clsConfig, "ViewForViewTypeIterator");
+
     auto clsLookNameIterator = 
         py::class_<LookNameIterator>(
             clsConfig, "LookNameIterator");
@@ -158,6 +178,12 @@ void bindPyConfig(py::module & m)
         .def(py::init(&Config::Create), 
              DOC(Config, Create))
 
+        .def("__deepcopy__", [](const ConstConfigRcPtr & self, py::dict)
+            {
+                return self->createEditableCopy();
+            },
+            "memo"_a)
+
         .def_static("CreateRaw", &Config::CreateRaw, 
                     DOC(Config, CreateRaw))
         .def_static("CreateFromEnv", &Config::CreateFromEnv, 
@@ -171,7 +197,11 @@ void bindPyConfig(py::module & m)
             }, 
              "str"_a, 
              DOC(Config, CreateFromStream))
-                    
+
+        .def_static("CreateFromBuiltinConfig", &Config::CreateFromBuiltinConfig, 
+                    DOC(Config, CreateFromBuiltinConfig))
+        .def_static("CreateFromConfigIOProxy", &Config::CreateFromConfigIOProxy,
+                    DOC(Config, CreateFromConfigIOProxy))
         .def("getMajorVersion", &Config::getMajorVersion, 
              DOC(Config, getMajorVersion))
         .def("setMajorVersion", &Config::setMajorVersion, "major"_a, 
@@ -288,6 +318,8 @@ void bindPyConfig(py::module & m)
              DOC(Config, addColorSpace))
         .def("removeColorSpace", &Config::removeColorSpace, "name"_a, 
              DOC(Config, removeColorSpace))
+        .def("isColorSpaceLinear", &Config::isColorSpaceLinear, "colorSpace"_a, "referenceSpaceType"_a, 
+             DOC(Config, isColorSpaceLinear))
         .def("isColorSpaceUsed", &Config::isColorSpaceUsed, "name"_a, 
              DOC(Config, isColorSpaceUsed))
         .def("clearColorSpaces", &Config::clearColorSpaces, 
@@ -302,6 +334,38 @@ void bindPyConfig(py::module & m)
              DOC(Config, setInactiveColorSpaces))
         .def("getInactiveColorSpaces", &Config::getInactiveColorSpaces, 
              DOC(Config, getInactiveColorSpaces))
+        .def("isInactiveColorSpace", &Config::isInactiveColorSpace, "colorspace"_a,
+             DOC(Config, isInactiveColorSpace))      
+        .def_static("IdentifyBuiltinColorSpace", [](const ConstConfigRcPtr & srcConfig,
+                                                    const ConstConfigRcPtr & builtinConfig,
+                                                    const char * builtinColorSpaceName)
+            {
+                return Config::IdentifyBuiltinColorSpace(srcConfig,
+                                                         builtinConfig,
+                                                         builtinColorSpaceName);
+            },
+                    "srcConfig"_a, "builtinConfig"_a, "builtinColorSpaceName"_a,
+                    DOC(Config, IdentifyBuiltinColorSpace))
+
+        .def_static("IdentifyInterchangeSpace", [](const ConstConfigRcPtr & srcConfig,
+                                                   const char * srcColorSpaceName,
+                                                   const ConstConfigRcPtr & builtinConfig,
+                                                   const char * builtinColorSpaceName)
+            {
+                const char * srcInterchangePtr = nullptr;
+                const char * builtinInterchangePtr = nullptr;
+
+                Config::IdentifyInterchangeSpace(&srcInterchangePtr,
+                                                 &builtinInterchangePtr,
+                                                 srcConfig,
+                                                 srcColorSpaceName,
+                                                 builtinConfig,
+                                                 builtinColorSpaceName);
+                // Return the tuple by value which copies the strings values.
+                return std::make_tuple(std::string(srcInterchangePtr), std::string(builtinInterchangePtr));
+            },
+                "srcConfig"_a, "srcColorSpaceName"_a, "builtinConfig"_a, "builtinColorSpaceName"_a,
+                DOC(Config, IdentifyInterchangeSpace))
 
         // Roles
         .def("setRole", &Config::setRole, "role"_a, "colorSpaceName"_a, 
@@ -316,6 +380,10 @@ void bindPyConfig(py::module & m)
             { 
                 return RoleColorSpaceIterator(self); 
             })
+        .def("getRoleColorSpace", 
+             (const char * (Config::*)(const char *) const) &Config::getRoleColorSpace, 
+             "roleName"_a,
+             DOC(Config, getRoleColorSpace)) 
 
         // Display/View Registration
         .def("addSharedView",
@@ -342,13 +410,28 @@ void bindPyConfig(py::module & m)
             { 
                 return DisplayIterator(self); 
             })
-        .def("getDefaultView", &Config::getDefaultView, "display"_a, 
+        .def("getDisplaysAll", [](ConfigRcPtr & self) 
+            { 
+                return DisplayAllIterator(self); 
+            })
+        .def("getDefaultView",
+             (const char * (Config::*)(const char *) const)
+             &Config::getDefaultView, "display"_a, 
+             DOC(Config, getDefaultView))
+        .def("getDefaultView",
+             (const char * (Config::*)(const char *, const char *) const)
+             &Config::getDefaultView, "display"_a, "colorSpacename"_a,
              DOC(Config, getDefaultView))
         .def("getViews", [](ConfigRcPtr & self, const std::string & display)
              {
                  return ViewIterator(self, display);
              },
              "display"_a)
+        .def("getViews", [](ConfigRcPtr & self, ViewType type, const std::string & display)
+             {
+                 return ViewForViewTypeIterator(self, type, display);
+             },
+             "type"_a, "display"_a)
         .def("getViews", [](ConfigRcPtr & self, 
                             const std::string & display, 
                             const std::string & colorSpaceName)
@@ -394,6 +477,59 @@ void bindPyConfig(py::module & m)
              DOC(Config, removeDisplayView))
         .def("clearDisplays", &Config::clearDisplays, 
              DOC(Config, clearDisplays))
+
+        // Virtual Display
+        .def("addVirtualDisplayView", &Config::addVirtualDisplayView, 
+             "view"_a, "viewTransformName"_a, "colorSpaceName"_a, 
+             "looks"_a = "",
+             "ruleName"_a = "", 
+             "description"_a = "", 
+             DOC(Config, addVirtualDisplayView))
+        .def("addVirtualDisplaySharedView", &Config::addVirtualDisplaySharedView, "sharedView"_a,
+             DOC(Config, addVirtualDisplaySharedView))
+        .def("getVirtualDisplayViews", [](ConfigRcPtr & self, ViewType type)
+             {
+                 return VirtualViewIterator(self, type);
+             },
+             "display"_a)
+        .def("getVirtualDisplayViewTransformName", &Config::getVirtualDisplayViewTransformName, 
+             "view"_a,
+             DOC(Config, getVirtualDisplayViewTransformName))
+        .def("getVirtualDisplayViewColorSpaceName", &Config::getVirtualDisplayViewColorSpaceName, 
+             "view"_a,
+             DOC(Config, getVirtualDisplayViewColorSpaceName))
+        .def("getVirtualDisplayViewLooks", &Config::getVirtualDisplayViewLooks, "view"_a,
+             DOC(Config, getVirtualDisplayViewLooks))
+        .def("getVirtualDisplayViewRule", &Config::getVirtualDisplayViewRule, "view"_a,
+             DOC(Config, getVirtualDisplayViewRule))
+        .def("getVirtualDisplayViewDescription", &Config::getVirtualDisplayViewDescription, 
+             "view"_a,
+             DOC(Config, getVirtualDisplayViewDescription))
+        .def("removeVirtualDisplayView", &Config::removeVirtualDisplayView, "view"_a,
+             DOC(Config, removeVirtualDisplayView))
+        .def("clearVirtualDisplay", &Config::clearVirtualDisplay,
+             DOC(Config, clearVirtualDisplay))
+        .def("instantiateDisplayFromMonitorName", &Config::instantiateDisplayFromMonitorName, 
+             "monitorName"_a,
+             DOC(Config, instantiateDisplayFromMonitorName))
+        .def("instantiateDisplayFromICCProfile", &Config::instantiateDisplayFromICCProfile, 
+             "ICCProfileFilepath"_a,
+             DOC(Config, instantiateDisplayFromICCProfile))
+        .def("isDisplayTemporary", [](ConfigRcPtr & self, const std::string & display) -> bool
+             {
+                 for (int i = 0; i < self->getNumDisplaysAll(); i++)
+                 {
+                     std::string other(self->getDisplayAll(i));
+                     if (StringUtils::Compare(display, other))
+                     {
+                         return self->isDisplayTemporary(i);
+                     }
+                 }
+                 return false;
+             },
+             "display"_a)
+
+        // Active Displays and Views
         .def("setActiveDisplays", &Config::setActiveDisplays, "displays"_a, 
              DOC(Config, setActiveDisplays))
         .def("getActiveDisplays", &Config::getActiveDisplays, 
@@ -459,7 +595,7 @@ void bindPyConfig(py::module & m)
         .def("clearViewTransforms", &Config::clearViewTransforms, 
              DOC(Config, clearViewTransforms))
 
-        // Named Transforms.
+        // Named Transforms
         .def("getNamedTransform", &Config::getNamedTransform, "name"_a)
 
         .def("getNamedTransformNames", [](ConfigRcPtr & self,
@@ -550,25 +686,70 @@ void bindPyConfig(py::module & m)
              &Config::getProcessor, 
              "context"_a, "srcColorSpaceName"_a, "display"_a, "view"_a, "direction"_a, 
              DOC(Config, getProcessor, 6))
+        .def("getProcessor", 
+             (ConstProcessorRcPtr (Config::*)(const ConstNamedTransformRcPtr &, 
+                                              TransformDirection) const) 
+             &Config::getProcessor, 
+             "namedTransform"_a, "direction"_a, 
+             DOC(Config, getProcessor, 7))
+        .def("getProcessor", 
+             (ConstProcessorRcPtr (Config::*)(const ConstContextRcPtr &, 
+                                              const ConstNamedTransformRcPtr &, 
+                                              TransformDirection) const) 
+             &Config::getProcessor, 
+             "context"_a, "namedTransform"_a, "direction"_a, 
+             DOC(Config, getProcessor, 8))
+        .def("getProcessor", 
+             (ConstProcessorRcPtr (Config::*)(const char *, 
+                                              TransformDirection) const) 
+             &Config::getProcessor, 
+             "namedTransformName"_a, "direction"_a,
+             DOC(Config, getProcessor, 9))
+        .def("getProcessor", 
+             (ConstProcessorRcPtr (Config::*)(const ConstContextRcPtr &, 
+                                              const char *,
+                                              TransformDirection) const)
+             &Config::getProcessor, 
+             "context"_a, "namedTransformName"_a, "direction"_a,
+             DOC(Config, getProcessor, 10))
         .def("getProcessor", 
              (ConstProcessorRcPtr (Config::*)(const ConstTransformRcPtr &) const) 
              &Config::getProcessor, 
              "transform"_a, 
-             DOC(Config, getProcessor, 7))
+             DOC(Config, getProcessor, 11))
         .def("getProcessor", 
              (ConstProcessorRcPtr (Config::*)(const ConstTransformRcPtr &, 
                                               TransformDirection) const) 
              &Config::getProcessor, 
              "transform"_a, "direction"_a, 
-             DOC(Config, getProcessor, 8))
+             DOC(Config, getProcessor, 12))
         .def("getProcessor", 
              (ConstProcessorRcPtr (Config::*)(const ConstContextRcPtr &, 
                                               const ConstTransformRcPtr &, 
                                               TransformDirection) const) 
              &Config::getProcessor, 
              "context"_a, "transform"_a, "direction"_a, 
-             DOC(Config, getProcessor, 9))
-
+             DOC(Config, getProcessor, 13))
+        .def_static("GetProcessorToBuiltinColorSpace", [](const ConstConfigRcPtr & srcConfig,
+                                                          const char * srcColorSpaceName,
+                                                          const char * builtinColorSpaceName)
+            {
+                return Config::GetProcessorToBuiltinColorSpace(srcConfig,
+                                                               srcColorSpaceName,
+                                                               builtinColorSpaceName);
+            },
+                    "srcConfig"_a, "srcColorSpaceName"_a, "builtinColorSpaceName"_a,
+                    DOC(Config, GetProcessorToBuiltinColorSpace))
+        .def_static("GetProcessorFromBuiltinColorSpace", [](const char * builtinColorSpaceName,
+                                                          ConstConfigRcPtr srcConfig,
+                                                          const char * srcColorSpaceName)
+            {
+                return Config::GetProcessorFromBuiltinColorSpace(builtinColorSpaceName,
+                                                                 srcConfig,
+                                                                 srcColorSpaceName);
+            },
+                    "builtinColorSpaceName"_a, "srcConfig"_a, "srcColorSpaceName"_a,
+                    DOC(Config, GetProcessorFromBuiltinColorSpace))
         .def_static("GetProcessorFromConfigs", [](const ConstConfigRcPtr & srcConfig,
                                                   const char * srcColorSpaceName,
                                                   const ConstConfigRcPtr & dstConfig,
@@ -630,10 +811,95 @@ void bindPyConfig(py::module & m)
                     "srcContext"_a, "srcConfig"_a, "srcColorSpaceName"_a, "srcInterchangeName"_a, 
                     "dstContext"_a, "dstConfig"_a, "dstColorSpaceName"_a, "dstInterchangeName"_a, 
                     DOC(Config, GetProcessorFromConfigs, 4))
-
+        .def_static("GetProcessorFromConfigs", [](const ConstConfigRcPtr & srcConfig,
+                                                  const char * srcColorSpaceName,
+                                                  const ConstConfigRcPtr & dstConfig,
+                                                  const char * dstDisplay,
+                                                  const char * dstView,
+                                                  TransformDirection direction)
+            {
+                return Config::GetProcessorFromConfigs(srcConfig, srcColorSpaceName,
+                                                       dstConfig, dstDisplay, dstView, direction);
+            },
+                    "srcConfig"_a, "srcColorSpaceName"_a, "dstConfig"_a, "dstDisplay"_a, "dstView"_a, "direction"_a,
+                    DOC(Config, GetProcessorFromConfigs, 5))
+        .def_static("GetProcessorFromConfigs", [](const ConstContextRcPtr & srcContext,
+                                                  const ConstConfigRcPtr & srcConfig,
+                                                  const char * srcColorSpaceName,
+                                                  const ConstContextRcPtr & dstContext,
+                                                  const ConstConfigRcPtr & dstConfig,
+                                                  const char * dstDisplay,
+                                                  const char * dstView,
+                                                  TransformDirection direction)
+            {
+                return Config::GetProcessorFromConfigs(srcContext, srcConfig, srcColorSpaceName,
+                                                       dstContext, dstConfig, dstDisplay, dstView, direction);
+            },
+                    "srcContext"_a, "srcConfig"_a, "srcColorSpaceName"_a,
+                    "dstContext"_a, "dstConfig"_a, "dstView"_a, "dstDisplay"_a, "direction"_a,
+                    DOC(Config, GetProcessorFromConfigs, 6))
+        .def_static("GetProcessorFromConfigs", [](const ConstConfigRcPtr & srcConfig,
+                                                  const char * srcColorSpaceName,
+                                                  const char * srcInterchangeName,
+                                                  const ConstConfigRcPtr & dstConfig,
+                                                  const char * dstDisplay,
+                                                  const char * dstView,
+                                                  const char * dstInterchangeName,
+                                                  TransformDirection direction)
+            {
+                return Config::GetProcessorFromConfigs(srcConfig,
+                                                       srcColorSpaceName,
+                                                       srcInterchangeName,
+                                                       dstConfig,
+                                                       dstDisplay,
+                                                       dstView,
+                                                       dstInterchangeName,
+                                                       direction);
+            },
+                    "srcConfig"_a, "srcColorSpaceName"_a, "srcInterchangeName"_a,
+                    "dstConfig"_a, "dstDisplay"_a, "dstView"_a, "dstInterchangeName"_a, "direction"_a,
+                    DOC(Config, GetProcessorFromConfigs, 7))
+        .def_static("GetProcessorFromConfigs", [](const ConstContextRcPtr & srcContext,
+                                                  const ConstConfigRcPtr & srcConfig,
+                                                  const char * srcColorSpaceName,
+                                                  const char * srcInterchangeName,
+                                                  const ConstContextRcPtr & dstContext,
+                                                  const ConstConfigRcPtr & dstConfig,
+                                                  const char * dstDisplay,
+                                                  const char * dstView,
+                                                  const char * dstInterchangeName,
+                                                  TransformDirection direction)
+            {
+                return Config::GetProcessorFromConfigs(srcContext,
+                                                       srcConfig,
+                                                       srcColorSpaceName,
+                                                       srcInterchangeName,
+                                                       dstContext,
+                                                       dstConfig,
+                                                       dstDisplay,
+                                                       dstView,
+                                                       dstInterchangeName,
+                                                       direction);
+            },
+                    "srcContext"_a, "srcConfig"_a, "srcColorSpaceName"_a, "srcInterchangeName"_a,
+                    "dstContext"_a, "dstConfig"_a, "dstDisplay"_a, "dstView"_a, "dstInterchangeName"_a, "direction"_a,
+                    DOC(Config, GetProcessorFromConfigs, 8))
         .def("setProcessorCacheFlags", &Config::setProcessorCacheFlags, "flags"_a, 
              DOC(Config, setProcessorCacheFlags))
-                
+        .def("clearProcessorCache", &Config::clearProcessorCache, 
+             DOC(Config, setProcessorCacheFlags))
+
+        // Archiving
+        .def("isArchivable", &Config::isArchivable, DOC(Config, isArchivable))
+        .def("archive", [](ConfigRcPtr & self, const char * filepath) 
+            {
+                std::ofstream f(filepath, std::ofstream::out | std::ofstream::binary);
+                self->archive(f);
+                f.close(); 
+            }, 
+            DOC(Config, archive))
+
+        // Conversion to string
         .def("__str__", [](ConfigRcPtr & self)
             {
                 std::ostringstream os;
@@ -844,6 +1110,20 @@ void bindPyConfig(py::module & m)
                 return it.m_obj->getDisplay(i);
             });
 
+    clsDisplayAllIterator
+        .def("__len__", [](DisplayAllIterator & it) { return it.m_obj->getNumDisplaysAll(); })
+        .def("__getitem__", [](DisplayAllIterator & it, int i) 
+            { 
+                it.checkIndex(i, it.m_obj->getNumDisplaysAll());
+                return it.m_obj->getDisplayAll(i);
+            })
+        .def("__iter__", [](DisplayAllIterator & it) -> DisplayAllIterator & { return it; })
+        .def("__next__", [](DisplayAllIterator & it)
+            {
+                int i = it.nextIndex(it.m_obj->getNumDisplaysAll());
+                return it.m_obj->getDisplayAll(i);
+            });
+
     clsSharedViewIterator
         .def("__len__", [](SharedViewIterator & it) { return it.m_obj->getNumViews(VIEW_SHARED,
                                                                                    nullptr); })
@@ -859,6 +1139,23 @@ void bindPyConfig(py::module & m)
                 return it.m_obj->getView(VIEW_SHARED, nullptr, i);
             });
 
+    clsVirtualViewIterator
+        .def("__len__", [](VirtualViewIterator & it)
+            { 
+                return it.m_obj->getVirtualDisplayNumViews(std::get<0>(it.m_args)); 
+            })
+        .def("__getitem__", [](VirtualViewIterator & it, int i)
+            { 
+                it.checkIndex(i, it.m_obj->getVirtualDisplayNumViews(std::get<0>(it.m_args)));
+                return it.m_obj->getVirtualDisplayView(std::get<0>(it.m_args), i);
+            })
+        .def("__iter__", [](VirtualViewIterator & it) -> VirtualViewIterator & { return it; })
+        .def("__next__", [](VirtualViewIterator & it)
+            {
+                int i = it.nextIndex(it.m_obj->getVirtualDisplayNumViews(std::get<0>(it.m_args)));
+                return it.m_obj->getVirtualDisplayView(std::get<0>(it.m_args), i);
+            });
+
     clsViewIterator
         .def("__len__", [](ViewIterator & it)
                         { return it.m_obj->getNumViews(std::get<0>(it.m_args).c_str()); })
@@ -885,7 +1182,10 @@ void bindPyConfig(py::module & m)
                 return it.m_obj->getView(std::get<0>(it.m_args).c_str(),
                                          std::get<1>(it.m_args).c_str(), i);
             })
-        .def("__iter__", [](ViewForColorSpaceIterator & it) -> ViewForColorSpaceIterator & { return it; })
+        .def("__iter__", [](ViewForColorSpaceIterator & it) -> ViewForColorSpaceIterator & 
+            { 
+                return it; 
+            })
         .def("__next__", [](ViewForColorSpaceIterator & it)
             {
                 int i = it.nextIndex(it.m_obj->getNumViews(std::get<0>(it.m_args).c_str(),
@@ -894,6 +1194,29 @@ void bindPyConfig(py::module & m)
                                          std::get<1>(it.m_args).c_str(), i);
             });
 
+    clsViewForViewTypeIterator
+        .def("__len__", [](ViewForViewTypeIterator & it)
+                        { return it.m_obj->getNumViews(std::get<0>(it.m_args),
+                                                       std::get<1>(it.m_args).c_str()); })
+        .def("__getitem__", [](ViewForViewTypeIterator & it, int i)
+            { 
+                it.checkIndex(i, it.m_obj->getNumViews(std::get<0>(it.m_args),
+                                                       std::get<1>(it.m_args).c_str()));
+                return it.m_obj->getView(std::get<0>(it.m_args),
+                                         std::get<1>(it.m_args).c_str(), i);
+            })
+        .def("__iter__", [](ViewForViewTypeIterator & it) -> ViewForViewTypeIterator & 
+            { 
+                return it; 
+            })
+        .def("__next__", [](ViewForViewTypeIterator & it)
+            {
+                int i = it.nextIndex(it.m_obj->getNumViews(std::get<0>(it.m_args),
+                                                           std::get<1>(it.m_args).c_str()));
+                return it.m_obj->getView(std::get<0>(it.m_args),
+                                         std::get<1>(it.m_args).c_str(), i);
+            });
+
     clsLookNameIterator
         .def("__len__", [](LookNameIterator & it) { return it.m_obj->getNumLooks(); })
         .def("__getitem__", [](LookNameIterator & it, int i) 
@@ -1054,6 +1377,12 @@ void bindPyConfig(py::module & m)
           DOC(PyOpenColorIO, GetCurrentConfig));
     m.def("SetCurrentConfig", &SetCurrentConfig, "config"_a, 
           DOC(PyOpenColorIO, SetCurrentConfig));
+
+    m.def("ExtractOCIOZArchive", &ExtractOCIOZArchive, 
+          DOC(PyOpenColorIO, ExtractOCIOZArchive));
+
+    m.def("ResolveConfigPath", &ResolveConfigPath, 
+          DOC(PyOpenColorIO, ResolveConfigPath));
 }
 
 } // namespace OCIO_NAMESPACE
